﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;
using System.Collections.Generic;
using System.Linq;
using System.Collections.ObjectModel;

namespace RadScheduleViewPerformance
{
	public class ViewModel : ViewModelBase
	{
		private ObservableCollection<Appointment> appointments;
		private IEnumerable<IResource> resources;
		private IEnumerable<IResourceType> resourceTypes;
		private int resourcesIndex;
		private int resourcesPerPage;

		public ViewModel()
		{
			this.resources = GenerateResources();

			var appointments = this.resources.SelectMany(this.GenerateAppointments);
			this.Appointments = new ObservableCollection<Appointment>(appointments);

			IResourceType resourceType = new ResourceType("Employee");
			resourceType.Resources.AddRange(this.resources);
			this.resourceTypes = resourceType.ToEnumerable().ToList();

			this.PreviousResources = new DelegateCommand(GetPreviousResources);
			this.NextResources = new DelegateCommand(GetNextResources);

			this.resourcesIndex = 0;
			this.ResourcesPerPage = 5;
		}

		public ObservableCollection<Appointment> Appointments
		{
			get
			{
				return this.appointments;
			}
			set
			{
				this.appointments = value;
				OnPropertyChanged("Appointments");
			}
		}

		public IEnumerable<IResource> Resources
		{
			get
			{
				return this.resources;
			}
		}

		public IEnumerable<IResourceType> ResourceTypes
		{
			get
			{
				return this.GetResources();
			}
		}

		public int ResourcesPerPage
		{
			get
			{
				return this.resourcesPerPage;
			}
			set
			{
				this.resourcesPerPage = value;
				OnPropertyChanged("ResourceTypes");
			}
		}

		public ICommand PreviousResources { get; private set; }
		public ICommand NextResources { get; private set; }
			
		private IEnumerable<IResource> GenerateResources()
		{
			var resources = new List<IResource>();

			for (int index = 1; index < 30; index++)
			{
				resources.Add(new Resource("Employee " + index, "Employee"));
			}
			return resources;
		}

		private IEnumerable<Appointment> GenerateAppointments(IResource resource)
		{
			var appointments =
				from i in Enumerable.Range(1, 15)
				select new Appointment()
				{
					Subject = "Appointment" + i,
					Start = DateTime.Now.Date.Add(new TimeSpan(i + 1, 0, 0)),
					End = DateTime.Now.Date.Add(new TimeSpan(i+2,0,0)),
					Resources = {resource},
				};

			return appointments;
		}

		private IEnumerable<IResourceType> GetResources()
		{
			int startIndex = this.resourcesIndex * this.ResourcesPerPage;
			int endIndex = startIndex + this.ResourcesPerPage;
			if (endIndex > this.resourceTypes.First().Resources.Count)
			{
				endIndex = this.resourceTypes.First().Resources.Count;
			}

			List<Resource> resources = new List<Resource>();
			for (int i = startIndex; i < endIndex; i++)
			{
				var resource = (Resource)this.resourceTypes.First().Resources[i];
				resources.Add(resource);
			}

			IResourceType resourceType = new ResourceType("Employee");
			resourceType.Resources.AddRange(resources);
			IEnumerable<IResourceType> result = resourceType.ToEnumerable();

			return result;
		}

		private void GetNextResources(object param)
		{
			int resourcesCount = this.resourceTypes.First().Resources.Count;
			int maxResourcesIndex = resourcesCount / this.ResourcesPerPage;
			if (resourcesCount % this.ResourcesPerPage == 0)
			{
				maxResourcesIndex--;
			}
			if (resourcesIndex < maxResourcesIndex)
			{
				resourcesIndex++;
				OnPropertyChanged("ResourceTypes");
			}
		}

		private void GetPreviousResources(object param)
		{
			int resourcesCount = this.resourceTypes.First().Resources.Count;

			if (this.resourcesIndex > 0)
			{
				resourcesIndex--;
				OnPropertyChanged("ResourceTypes");
			}
		}
	}
}
